local Recount2 = Recount2

local tinsert = table.insert
local tremove = table.remove

Recount2.Configs = {}

local Recount2_Configs = Recount2.Configs

local function SetFontSize(string, size)
	local Font, Height, Flags = string:GetFont()
	string:SetFont(Font, size, Flags)
end

local function UpdateConfig(self)
	local inactiveFilters = {}
	self.activeFilters:RemoveEntries()
	self.inactiveFilters:RemoveEntries()

	for k, v in ipairs(Recount2.SharedData.Filters) do
		tinsert(inactiveFilters, v)
	end

	for k, v in ipairs(self.configuring.filters) do
		self.activeFilters:AddEntry(v.name, v)

		for k2, v2 in ipairs(inactiveFilters) do
			if v==v2 then
				tremove(inactiveFilters, k2)
				break
			end
		end
	end

	for k, v in ipairs(inactiveFilters) do
		self.inactiveFilters:AddEntry(v.name, v)
	end
end

local function AddNewView(self)
	local filter = {
		name = 'New View',
		barValue = 'cTotal',
		leftText = '[cPos, cName:$%d. %s]',
		rightText = '[cTotal,  100*cTotal/lTotal:$%d (%.1f%%)]',
		dataSource = 'Overall',
		infoFilter = Recount2.BitFlags.Info['Damage'],}
	
	tinsert(Recount2.SharedData.Filters, filter)
	tinsert(self.window.configuring.filters, filter)

	self.window:UpdateConfig()
end

local function MoveToActive(self)
	local window = self.window
	local selected = window.inactiveFilters:GetSelected()
	if selected then
		tinsert(window.configuring.filters, selected)
		window:UpdateConfig()
	end
end

local function MoveToInactive(self)
	local window = self.window
	local selected = window.activeFilters:GetSelected()
	if selected then
		for k, v in ipairs(window.configuring.filters) do
			if v==selected then
				tremove(window.configuring.filters, k)
				window:UpdateConfig()
				return
			end
		end
	end
end

local function EditActive(self)
	local selected = self.window.activeFilters:GetSelected()

	if selected then
		Recount2.Configs:ConfigView(selected)
	end
end

local function EditInactive(self)
	local selected = self.window.inactiveFilters:GetSelected()

	if selected then
		Recount2.Configs:ConfigView(selected)
	end
end

local function RemoveView(self)
	local selected = self.window.inactiveFilters:GetSelected()

	if selected then
		for k, v in ipairs(Recount2.SharedData.Filters) do
			if v==selected then
				tremove(Recount2.SharedData.Filters, k)
				self.window:UpdateConfig()
				return
			end
		end
	end	
end

local function AddViewTab(window, innerFrame, tabs)
	local tab = tabs:AddTab("Views")

	window.activeFilters = Recount2.Widgets:CreateScrollList(tab, 165, 196, 14, 1)
	window.activeFilters:SetPoint("TOPLEFT", innerFrame, "TOPLEFT", 1, -23)

	window.inactiveFilters = Recount2.Widgets:CreateScrollList(tab, 165, 196, 14, 1)
	window.inactiveFilters:SetPoint("TOPRIGHT", innerFrame, "TOPRIGHT", -1, -23)

	window.moveToActive = Recount2.Widgets:CreateButton(tab, 55, 16, "<-", MoveToActive)
	window.moveToActive:SetPoint("TOP", innerFrame, "TOP", 0, -100)
	window.moveToActive.window = window

	window.moveToInactive = Recount2.Widgets:CreateButton(tab, 55, 16, "->", MoveToInactive)
	window.moveToInactive:SetPoint("TOP", window.moveToActive, "BOTTOM", 0, -4)
	window.moveToInactive.window = window

	window.newToActive = Recount2.Widgets:CreateButton(tab, 81, 16, "New", AddNewView)
	window.newToActive:SetPoint("TOPLEFT", window.activeFilters, "BOTTOMLEFT", 0, -4)
	window.newToActive.window = window

	window.editFromActive = Recount2.Widgets:CreateButton(tab, 81, 16, "Edit", EditActive)
	window.editFromActive:SetPoint("TOPRIGHT", window.activeFilters, "BOTTOMRIGHT", 0, -4)
	window.editFromActive.window = window

	window.removeFromInactive = Recount2.Widgets:CreateButton(tab, 81, 16, "Remove", RemoveView)
	window.removeFromInactive:SetPoint("TOPLEFT", window.inactiveFilters, "BOTTOMLEFT", 0, -4)
	window.removeFromInactive.window = window

	window.editFromInactive = Recount2.Widgets:CreateButton(tab, 81, 16, "Edit", EditInactive)
	window.editFromInactive:SetPoint("TOPRIGHT", window.inactiveFilters, "BOTTOMRIGHT", 0, -4)
	window.editFromInactive.window = window
end

local function CreateLabel(parent, text)
	local label = parent:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
	label:SetJustifyH("LEFT")
	label:SetText(text)
	label:SetTextColor(1,1,0.2,1)
	Recount2:AddFontString(label)
	SetFontSize(label, 13)
	return label
end

local function CreateCheckboxList(tab, innerFrame, title, constLookups, translations, xPos, yPos)
	local label = CreateLabel(tab, title)
	label:SetPoint("TOPLEFT", innerFrame, "TOPLEFT", xPos, -20+yPos)

	for i, text in ipairs(constLookups) do
		 local checkbox= Recount2.Widgets:CreateCheckboxList(tab, 75, 14, translations[text], "Main_"..title, text)
		 checkbox:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, -(i-1)*14)
	end
end

--Tables
local consts = Recount2.Const
local translations = Recount2.TranslationTables

local function UpdateFilterEvents()
	Recount2.filterEvents = Recount2.Widgets:GetListCheckValue("Main_Event Types")
	Recount2PerCharDB.filterEvents = Recount2.filterEvents
end

local function UpdateFilterAffil()
	Recount2.filterAffiliation = Recount2.Widgets:GetListCheckValue("Main_Affiliation")
	Recount2PerCharDB.filterAffiliation = Recount2.filterAffiliation
end

local function AddFilterTab(window, innerFrame, tabs)
	local tab = tabs:AddTab("Filtering")

	CreateCheckboxList(tab, innerFrame, "Event Types", consts.EventTypes, translations.EventTypes, 0, 0)
	CreateCheckboxList(tab, innerFrame, "Affiliation", consts.Affiliation, translations.Affiliation, 79, 0)

	Recount2.Widgets:SetCheckUpdateFunc("Main_Event Types", UpdateFilterEvents, {})
	Recount2.Widgets:SetCheckUpdateFunc("Main_Affiliation", UpdateFilterAffil, {})

	warnLabel = CreateLabel(tab, "Warning these filters effect which events get processed not what is displayed")
	warnLabel:SetPoint('BOTTOMLEFT', innerFrame, 'BOTTOMLEFT', 4, 4)
end

local function AddAppearanceTab(window, innerFrame, tabs)
	local tab = tabs:AddTab("Appearance")
end

function Recount2_Configs:ConfigMainList(name)
	local window = Recount2.configListWindows[name]

	if window == nil then
		window=Recount2.Widgets:CreateWindow("Recount2_ConfigMainListWindow_"..name, 400, 250+20)
		window:SetPoint("CENTER", UIParent, "CENTER")

		Recount2.configListWindows[name] = window

		window.Tabs = Recount2.Widgets:CreateTabs(window.innerFrame, window.innerFrame:GetWidth()+2, 16)
		window.Tabs:SetPoint("TOP", window.innerFrame, "TOP")
		
		AddViewTab(window, window.innerFrame, window.Tabs)
		AddFilterTab(window, window.innerFrame, window.Tabs)
		
		window.UpdateConfig = UpdateConfig
		window:SetTitle("Configuring List Window - "..name)
		window:MakeUnresizable()
	end

	window.configuring = Recount2.mainListWindows[name]
	window:UpdateConfig()

	Recount2.Widgets:SetListCheckValue("Main_Event Types", Recount2.filterEvents)
	Recount2.Widgets:SetListCheckValue("Main_Affiliation", Recount2.filterAffiliation)

	window:Show()
end